/* 
    kleds for KDE2/KDE3

    Copyright (C) 2001 Hans Matzen (hans@hansmatzen.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
extern "C" {
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
}
#include <khelpmenu.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kapp.h>
#include <dcopclient.h>
#include <qdatastream.h>

#include "kleds.h"
#include "prefs.h"

// constructor
KLeds::KLeds(QWidget *parent, const char *name) 
    : KMainWindow (parent, name), 
      DCOPObject("kleds")
{
    // read configuration
    config = KGlobal::config();
    this->read_config();
    
    // create new dockwidget
    dockw = new dockwidget(this);
    

    // configure dockwidget
    config_dockwidget();


    // connect mouse events to me
    // for the popups
    connect(dockw, SIGNAL(mouseClick(QMouseEvent *)), 
	    this, SLOT(activateMenu(QMouseEvent *)));

    setMinimumSize(QSize(32,32));

    // start daemon
    if (system ("kledsd") == -1)
	printf("start of kledsd failed\n");
}

void KLeds::read_config()
{
    config->setGroup("Layout");
    
    num_shape = (KLed::Shape) 
	config->readNumEntry("NUM_SHAPE", KLed::Circular);
    caps_shape= (KLed::Shape) 
	config->readNumEntry("CAPS_SHAPE", KLed::Circular);
    scroll_shape= (KLed::Shape) 
	config->readNumEntry("SCROLL_SHAPE", KLed::Circular);
    
    label_position = (uint) config->readNumEntry("LABEL_POSITION", LEFT);
    
    QColor *defcol=new QColor(0,255,0);
    num_color = config->readColorEntry("NUM_COLOR", defcol);
    caps_color = config->readColorEntry("CAPS_COLOR", defcol);
    scroll_color = config->readColorEntry("SCROLL_COLOR", defcol);
    bg_typ = (uint) config->readNumEntry("BG_TYP", 0);
    bg_color = config->readColorEntry("BG_COLOR", defcol);
    font = config->readFontEntry("FONT",new QFont("Helvetica", 8));
}

void KLeds::write_config()
{
    config->setGroup("Layout");
    
    config->writeEntry("NUM_SHAPE", num_shape);
    config->writeEntry("CAPS_SHAPE", caps_shape);
    config->writeEntry("SCROLL_SHAPE", scroll_shape);
    config->writeEntry("LABEL_POSITION", label_position);
    
    config->writeEntry("NUM_COLOR", num_color);
    config->writeEntry("CAPS_COLOR", caps_color);
    config->writeEntry("SCROLL_COLOR", scroll_color);
    config->writeEntry("BG_TYP", bg_typ);
    config->writeEntry("BG_COLOR", bg_color);

    config->writeEntry("FONT",font);

    config->sync();
}

// destructor
KLeds::~KLeds()
{
    // kill the daemon
    kill(daemon_pid,SIGTERM);
    // delete the dock widget
    delete dockw;
}

// send the new indicator state to the docked widget
void KLeds::state_changed(int state)
{
    dockw->set_LEDS(state);
}

// receive the pid of the daemon, to kill him later
void KLeds::get_pid(int pid)
{
    daemon_pid=pid;
}

void KLeds::activateMenu(QMouseEvent *e)
{
    KHelpMenu   *_hmenu;
    KPopupMenu  *_menu;
    enum MenuOp { mPref = 11, mQuit = 12};

    if (e->button() == RightButton) {
        // prepare helpmenu
        _hmenu = new KHelpMenu (this,kapp->aboutData(),false);
        _menu = _hmenu->menu();
	
        // setup popup menu
        _menu->insertSeparator();
        _menu->insertItem(i18n("Preferences..."), mPref);
        _menu->insertItem(i18n("Quit KLeds"), mQuit);
        _menu->adjustSize();
	_menu->move(e->globalPos());

	// execute popup and check result
	int i;
	i=_menu->exec();
	switch(i) {
            case mPref:
                this->preferences();
                break;
            case mQuit:
                this->~KLeds();
                kapp->quit();
		break;
            default:
                break;
        }
    }
}

void KLeds::preferences()
{
 
    Prefs *pd=new Prefs(this,"KLeds Preferences",TRUE);

    // set current values before showing prefs dialog
    pd->set_num_shape(num_shape);
    pd->set_caps_shape(caps_shape);
    pd->set_scroll_shape(scroll_shape);
    pd->set_num_color(num_color);
    pd->set_caps_color(caps_color);
    pd->set_scroll_color(scroll_color);
    pd->set_label_position(label_position);
    pd->set_bg_color(bg_typ,bg_color);
    pd->set_font(font);

    if ( pd->exec() == QDialog::Accepted ) {
	// get new values and write them to config file
	num_shape = pd->get_num_shape();
	caps_shape= pd->get_caps_shape();
	scroll_shape = pd->get_scroll_shape();
	num_color= pd->get_num_color();
	caps_color= pd->get_caps_color();
	scroll_color= pd->get_scroll_color();
	label_position= pd->get_label_position();
	bg_color= pd->get_bg_color();
	bg_typ = pd->get_bg_typ();
	font = pd->get_font();

	this->write_config();
	config_dockwidget();
    }
     
    delete pd;
}

void KLeds::config_dockwidget()
{

    // configure dockwidget
    dockw->config_numled(num_shape,num_color);
    dockw->config_capsled(caps_shape,caps_color);
    dockw->config_scrollled(scroll_shape,scroll_color);
    dockw->config_layout(label_position);
    dockw->config_background(bg_typ, bg_color);
    dockw->config_font(font);
}
